"""
Dataframe
"""
from typing import Dict, List, Tuple, Union

import pandas as pd

from . import api
from . import search
from .ts import time_serie


def dataframe(ts_codes: Union[int, List, Tuple], start: str, end: str) -> pd.DataFrame:
    """
    Creates a dataframe from a list of time serie codes.

    :param ts_codes: single code or list/tuple of time series codes.
    :param start: start date (DD/MM/YYYY).
    :param end: end date (DD/MM/YYYY).

    :return: Pandas dataframe.
    :rtype: pandas.DataFrame_

    Usage::

        >>> CDI = 12
        >>> INCC = 192  #  National Index of Building Costs
        >>> df = sgs.dataframe([CDI, INCC], start='02/01/2018', end='31/12/2018')
        >>> df.head()
                         12    192
        2018-01-01       NaN  0.31
        2018-01-02  0.026444   NaN
        2018-01-03  0.026444   NaN
        2018-01-04  0.026444   NaN
        2018-01-05  0.026444   NaN

    """
    if isinstance(ts_codes, int):
        ts_codes = [ts_codes]

    series = []
    for code in ts_codes:
        ts = time_serie(code, start, end)
        series.append(ts)

    df = pd.concat(series, axis=1)
    return df



"""
Shared functions.
"""
from datetime import datetime
import locale
from typing import Union


LRU_CACHE_SIZE = 32
MAX_ATTEMPT_NUMBER = 5


def to_datetime(date_string: str, language: str) -> Union[datetime, str]:
    """ Converts a date string to a datetime object """
    locales = {"pt": "pt_BR.utf-8", "en": "en_US.utf-8"}

    locale.setlocale(locale.LC_TIME, locales[language])

    dd_mm_aaaa = "%d/%m/%Y"
    mmm_aaaa = "%b/%Y"
    try:
        date = datetime.strptime(date_string, dd_mm_aaaa)
    except ValueError:
        try:
            date = datetime.strptime(date_string, mmm_aaaa)
        except ValueError:
            return date_string  # ignore errors and return original value
    return date



"""
# Time Series Management System

The SGS is a system maintened by the Central Bank of Brazil with the
objective of consolidating and making available economic-financial
information, as well as maintaining uniformity among documents
produced based on time series stored in it.
"""

from .__version__ import __version__
from .dataframe import dataframe
from .ts import time_serie
from .metadata import metadata
from .search import search_ts



import pytest
import requests

from sgs.search import *
from sgs.common import to_datetime

@pytest.mark.search
@pytest.mark.parametrize("language", ["en", "pt"])
def test_init_search_session(language):
    session = init_search_session(language)
    assert isinstance(session, requests.Session)


@pytest.mark.search
def test_search_by_code_english():
    code = 4
    results = search_ts(code, Language.en.value)
    print(results)
    metadata = results[0]
    assert metadata['name'] == "BM&F Gold - gramme"
    assert metadata['first_value'] == to_datetime("29/12/1989", "en")
    assert metadata['frequency'] == "D"


@pytest.mark.search
def test_search_by_code_portuguese():
    code = 4
    results = search_ts(code, Language.pt.value)
    metadata = results[0]
    assert metadata['name'] == "Ouro BM&F - grama"
    assert metadata['first_value'] == to_datetime("29/12/1989", "pt")
    assert metadata['frequency'] == "D"


@pytest.mark.search
@pytest.mark.parametrize(
    "query,language,expected",
    [
        (
            4,
            "pt",
            {
                "name": "Ouro BM&F - grama",
                "first_value": to_datetime("29/12/1989", "pt"),
                "freq": "D",
            },
        ),
        (
            4,
            "en",
            {
                "name": "BM&F Gold - gramme",
                "first_value": to_datetime("29/12/1989", "en"),
                "freq": "D",
            },
        ),
        (
            28209,
            "pt",
            {
                "name": (
                    "Saldo de títulos de dívida emitidos por "
                    "empresas e famílias - títulos privados"
                ),
                "first_value": to_datetime("01/01/2013", "pt"),
                "freq": "M",
            },
        ),
    ],
)
def test_search_by_code(query, language, expected):
    results = search_ts(query, language)
    assert isinstance(results, list)
    results = results[0]
    assert results["frequency"] == expected["freq"]
    assert results["name"] == expected["name"]
    first_value = expected["first_value"]
    assert results["first_value"] == first_value


@pytest.mark.search
def test_search_by_text():
    results = search_ts("Ouro BM$F - grama", "pt")
    assert isinstance(results, list)
    assert len(results) == 1

    # portuguese query and english language returns None
    results = search_ts("Ouro BM$F - grama", "en")
    assert results is None


@pytest.mark.search
def test_search_by_text_query_returns_multiple_result_pages():
    results = search_ts("produto", "pt")
    assert isinstance(results, list)
    assert len(results) == 50


@pytest.mark.search
def test_search_by_text_multiple_results():
    results = search_ts("dolar", "pt")
    assert isinstance(results, list)
    result_count = 41
    assert len(results) == result_count
    assert results[0]["code"] == 1
    assert results[-1]["code"] == 21636
